@@ -15,7 +15,7 @@ bootdata.trans.msm <- function(x) {
   colnames(data.boot) <- c(subj.name, time.name, state.name, x$data$covlabels)
   data.boot[,state.name] <- as.vector(rbind(x$data$state[inds], x$data$state[inds+1]))
   # in the bootstrap data, label each transition as being from a different subject
-  data.boot[,subj.name] <- rep(seq(along=inds), each=2) 
+  data.boot[,subj.name] <- rep(seq(along=inds), each=2)
   data.boot[,time.name] <- as.vector(rbind(x$data$time[inds], x$data$time[inds+1]))
   for (j in x$data$covlabels) {
     data.boot[seq(1, 2*length(inds)-1, 2), j] <- x$data$cov[inds, j] + x$data$covdata$covmeans[j]
@@ -54,20 +54,20 @@ bootdata.subject.msm <- function(x) {
 
 ### Given a fitted msm model, draw a bootstrap dataset, refit the
 ### model, and optionally compute a statistic on the refitted model.
-### Repeat B times, store the results in a list. 
+### Repeat B times, store the results in a list.
 ### msm objects tend to be large, so it is advised to compute a statistic on them by specifying "stat", instead
-### of using this function to return a list of refitted msm objects. 
+### of using this function to return a list of refitted msm objects.
 ### To compute more than one statistic, specify, e.g. stat=function(x)list(stat1(x),stat2(x))
 
-### Some of the arguments to the msm call might be user-defined objects. 
+### Some of the arguments to the msm call might be user-defined objects.
 ### e.g. qmatrix, ematrix, hmodel, ...
-### Put in help file that these must be in the working environment. 
+### Put in help file that these must be in the working environment.
 
 boot.msm <- function(x, stat=pmatrix.msm, B=500, file=NULL){
   boot.list <- vector(B, mode="list")
   for (i in 1:B) {
     boot.data <- if (x$hmodel$hidden || x$cmodel$ncens) bootdata.subject.msm(x) else bootdata.trans.msm(x)
-    x$call$data <- substitute(boot.data)    
+    x$call$data <- substitute(boot.data)
     boot.list[[i]] <- try(eval(x$call))
     if (!is.null(stat))
       boot.list[[i]] <- stat(boot.list[[i]])
@@ -76,8 +76,8 @@ boot.msm <- function(x, stat=pmatrix.msm, B=500, file=NULL){
   boot.list
 }
 
-### Utilities for calculating CIs for particular statistics e.g. pmatrix. 
-### Possibly do also for expected prevalences, or total length of stay? 
+### Utilities for calculating CIs for particular statistics e.g. pmatrix.
+### Possibly do also for expected prevalences
 
 pmatrix.ci.msm <- function(x, t, covariates="mean", cl=0.95, B=500) {
   p.list <- boot.msm(x, function(x)pmatrix.msm(x, t, covariates), B)
@@ -91,3 +91,26 @@ totlos.ci.msm <- function(x, start=1, fromt=0, tot=Inf, covariates="mean", cl=0.
   t.array <- do.call("rbind", t.list)
   apply(t.array, 2, function(x)(quantile(x, c(0.5 - cl/2, 0.5 + cl/2))))
 }
+
+expected.ci.msm <- function(x,
+                            times=NULL,
+                            timezero=NULL,
+                            initstates=NULL,
+                            covariates="mean",
+                            misccovariates="mean",
+                            piecewise.times=NULL,
+                            piecewise.covariates=NULL,
+                            risk=NULL,
+                            cl=0.95, B=500) {
+    if(is.null(risk)) risk <- observed.msm(x)$risk
+    e.list <- boot.msm(x, function(x){
+        expected.msm(x, times, timezero, initstates, covariates, misccovariates, piecewise.times, piecewise.covariates, risk)
+    }, B)
+    e.tab.array <- array(unlist(lapply(e.list, function(x)x[[1]])), dim=c(dim(e.list[[1]][[1]]), length(e.list)))
+    e.perc.array <- array(unlist(lapply(e.list, function(x)x[[2]])), dim=c(dim(e.list[[1]][[2]]), length(e.list)))
+    e.tab.ci <- apply(e.tab.array, c(1,2), function(x)(quantile(x, c(0.5 - cl/2, 0.5 + cl/2))))
+    e.perc.ci <- apply(e.perc.array, c(1,2), function(x)(quantile(x, c(0.5 - cl/2, 0.5 + cl/2))))
+    res <- list(aperm(e.tab.ci, c(2,3,1)),  aperm(e.perc.ci, c(2,3,1)))
+    names(res) <- c("Expected", "Expected percentages")
+    res
+}
